import { execSync } from 'child_process';
import { tmpdir as tmpdir$1 } from 'os';
import path from 'path';
import fs from 'fs';
import { v4 } from 'uuid';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

/**
 * @link https://github.com/sindresorhus/temp-dir/blob/main/index.js#L9
 */
const tmpdir = fs.realpathSync(tmpdir$1());
let tsconfigPathsAvailable;
const isTsconfigPathsAvailable = () => {
    if (typeof tsconfigPathsAvailable === 'boolean') {
        return tsconfigPathsAvailable;
    }
    try {
        // eslint-disable-next-line node/no-extraneous-require
        tsconfigPathsAvailable = !!require.resolve('tsconfig-paths');
    }
    catch (_a) {
        /**
         * `require.resolve` can not be mocked to fail
         * @link https://github.com/facebook/jest/issues/9543
         */
        /* istanbul ignore next */
        tsconfigPathsAvailable = false;
    }
    return tsconfigPathsAvailable;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const syncFnCache = new Map();
const createSyncFn = (workerPath) => {
    if (!path.isAbsolute(workerPath)) {
        throw new Error('`workerPath` must be absolute');
    }
    const cachedSyncFn = syncFnCache.get(workerPath);
    if (cachedSyncFn) {
        return cachedSyncFn;
    }
    let resolvedWorkerPath = workerPath;
    if (!resolvedWorkerPath.endsWith('.ts')) {
        resolvedWorkerPath = require.resolve(workerPath);
    }
    const executor = resolvedWorkerPath.endsWith('.ts')
        ? 'ts-node' +
            (isTsconfigPathsAvailable()
                ? ' -r tsconfig-paths/register'
                : /* istanbul ignore next */ '')
        : 'node';
    const syncFn = (...args) => {
        const filename = path.resolve(tmpdir, `synckit-${v4()}.json`);
        fs.writeFileSync(filename, JSON.stringify(args));
        const command = `${executor} ${resolvedWorkerPath} ${filename}`;
        try {
            execSync(command, {
                stdio: 'inherit',
            });
            const result = fs.readFileSync(filename, 'utf8');
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return JSON.parse(result);
        }
        finally {
            fs.unlinkSync(filename);
        }
    };
    syncFnCache.set(workerPath, syncFn);
    return syncFn;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const runAsWorker = (fn) => __awaiter(void 0, void 0, void 0, function* () {
    const filename = process.argv[2];
    const content = fs.readFileSync(filename, 'utf-8');
    const options = JSON.parse(content);
    const result = yield fn(...options);
    fs.writeFileSync(filename, JSON.stringify(result));
});

export { createSyncFn, runAsWorker, tmpdir };
