import { execSync } from 'child_process';
import { tmpdir as tmpdir$1 } from 'os';
import path from 'path';
import fs from 'fs';
import { v4 } from 'uuid';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

/**
 * @link https://github.com/sindresorhus/temp-dir/blob/main/index.js#L9
 */
var tmpdir = fs.realpathSync(tmpdir$1());
var tsconfigPathsAvailable;
var isTsconfigPathsAvailable = function () {
    if (typeof tsconfigPathsAvailable === 'boolean') {
        return tsconfigPathsAvailable;
    }
    try {
        // eslint-disable-next-line node/no-extraneous-require
        tsconfigPathsAvailable = !!require.resolve('tsconfig-paths');
    }
    catch (_a) {
        /**
         * `require.resolve` can not be mocked to fail
         * @link https://github.com/facebook/jest/issues/9543
         */
        /* istanbul ignore next */
        tsconfigPathsAvailable = false;
    }
    return tsconfigPathsAvailable;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
var syncFnCache = new Map();
var createSyncFn = function (workerPath) {
    if (!path.isAbsolute(workerPath)) {
        throw new Error('`workerPath` must be absolute');
    }
    var cachedSyncFn = syncFnCache.get(workerPath);
    if (cachedSyncFn) {
        return cachedSyncFn;
    }
    var resolvedWorkerPath = workerPath;
    if (!resolvedWorkerPath.endsWith('.ts')) {
        resolvedWorkerPath = require.resolve(workerPath);
    }
    var executor = resolvedWorkerPath.endsWith('.ts')
        ? 'ts-node' +
            (isTsconfigPathsAvailable()
                ? ' -r tsconfig-paths/register'
                : /* istanbul ignore next */ '')
        : 'node';
    var syncFn = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var filename = path.resolve(tmpdir, "synckit-" + v4() + ".json");
        fs.writeFileSync(filename, JSON.stringify(args));
        var command = executor + " " + resolvedWorkerPath + " " + filename;
        try {
            execSync(command, {
                stdio: 'inherit',
            });
            var result = fs.readFileSync(filename, 'utf8');
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return JSON.parse(result);
        }
        finally {
            fs.unlinkSync(filename);
        }
    };
    syncFnCache.set(workerPath, syncFn);
    return syncFn;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
var runAsWorker = function (fn) { return __awaiter(void 0, void 0, void 0, function () {
    var filename, content, options, result;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                filename = process.argv[2];
                content = fs.readFileSync(filename, 'utf-8');
                options = JSON.parse(content);
                return [4 /*yield*/, fn.apply(void 0, options)];
            case 1:
                result = _a.sent();
                fs.writeFileSync(filename, JSON.stringify(result));
                return [2 /*return*/];
        }
    });
}); };

export { createSyncFn, runAsWorker, tmpdir };
